function [Vx Vy] = PhaseAdjustAffine(P1_r,P2_r,P1_s,P2_s,fx,fy,window)

%PhaseAdjustAffine  Affine parameter estimation based on
%single-orthant analytic signal phases

%   [Vx Vy] = PhaseAdjustAffine(P1_r,P2_r,P1_s,P2_s,fx,fy,window) computes
%   the 2D estimated affine parameters (6 parameters) using phase adjustement
%   and two least square estimators, between two images with 2D spatial
%   modulations
%   Inputs : P1_r,P2_r phases of 2D single-orthant analytic signals
%               corresponding to the first image (generated using ashahn
%               function)
%            P1_s,P2_s phases of 2D single-orthant analytic signals
%               corresponding to the second image (generated using ashahn
%               function)
%            fx,fy carrier frequencies along each direction x and y
%            window window type used with least square estimators
%   Outputs : Vx,Vy estimated affine parameters (3 values for each spatial
%               direction

%   Authors: Florian Douziech, Adrian Basarab, may 2011.


% References that must be cited to refer to this work:
%   [1] F.Douziech, H. Liebgott, Olivier Bernard, Adrian Basarab
%   Affine phase based motion estimation applied to echocardiography,
%   IEEE Ultrasonics Symposium, 2011.
%
%   [2] Martino Alessandrini, Adrian Basarab, Loic Boussel, Xinxin Guo,
%   Andr Serusclat, Denis Friboulet, Denis Kouam, Olivier Bernard, Herv Liebgott,
%   A New Technique for the Estimation of Cardiac Motion in Echocardiography
%   Based on Transverse Oscillations: a preliminary evaluation in silico and
%   a feasibility demonstration in vivo, IEEE Transactions on Medical Imaging, 2014

% Example
% 
% x = 1:21; y = 1:21; [X,Y] = meshgrid(x,y);
% centerX = x(end)/2;         
% centerY = y(end)/2;         
% sigmax = 80; 
% sigmay = 120; 
% fx = 0.12; fy = 0.12;
% % True affine model
% u0 = 0.3; ux = 0.09; uy =0.01;;
% v0 = 0.5; vx = 0.02; vy =0.05; 
% % True motion field
% dx = ux*(X-centerX) + uy*(Y-centerY)+u0;
% dy = vx*(X-centerX) + vy*(Y-centerY)+v0;
% r = cos(2*pi*fx*X) .* cos(2*pi*fy*Y) .*...
% exp(-pi*((X-centerX)/sigmax).^2).*exp(-pi*((Y-centerY)/sigmay).^2);
% s = cos(2*pi*fx*(X-dx)) .* cos(2*pi*fy*(Y-dy)) .*...
% exp(-pi*(((X-dx)-centerX)/sigmax).^2).*exp(-pi*(((Y-dy)-centerY)/sigmay).^2);
% [r1,r2] = ashahn(r); [s1,s2] = ashahn(s);
% P1_r=atan2(imag(r1),real(r1));
% P2_r=atan2(imag(r2),real(r2));
% P1_s=atan2(imag(s1),real(s1));
% P2_s=atan2(imag(s2),real(s2));
% figure; imagesc(x,y,r); title('r')
% figure; imagesc(x,y,s); title('s')
% 
% [Vx_estim Vy_estim] = PhaseAdjustAffine(P1_r,P2_r,P1_s,P2_s,fx,fy,'rectwin')




if isempty(window)
    window = 'rectwin';
end

%%
[y_Bmax x_Bmax] =size(P1_r);

phi_diff1 = P1_r - P1_s;
phi_diff2 = P2_r - P2_s;

% threshold to eliminate phase jumps
seuil=pi;

model_y=(phi_diff1+phi_diff2);
model_x=(phi_diff1-phi_diff2);

chaine1=[window, '(x_Bmax)'];
chaine2=[window, '(y_Bmax)'];
windowHx=eval(chaine1);
windowHy=eval(chaine2);

windowH=windowHy*windowHx';

x_block=-(fix(x_Bmax/2)):(fix(x_Bmax/2));
y_block=-(fix(y_Bmax/2)):(fix(y_Bmax/2));
[Xe Ye]=meshgrid(x_block,y_block);

% eliminate phase jumps without phase unwrapping
ind_ssaut1=(abs(model_y)<seuil);
ind_ssaut2=(abs(model_x)<seuil);

window1=windowH(ind_ssaut1);
x_block1=Xe(ind_ssaut1).*window1;
y_block1=Ye(ind_ssaut1).*window1;

window2=windowH(ind_ssaut2);
x_block2=Xe(ind_ssaut2).*window2;
y_block2=Ye(ind_ssaut2).*window2;

model1=model_y(ind_ssaut1).*window1;
model2=model_x(ind_ssaut2).*window2;

%% Least square estimator
B1=[window1 x_block1 y_block1];
BB1=inv(B1'*B1);
Vy = BB1 *B1'*model1;

B2=[window2 x_block2 y_block2];
BB2=inv(B2'*B2);
Vx = BB2 *B2'*model2;

%% estimated affine parameters
Vy = (1/(4*pi*fy)).*Vy;
Vx = (1/(4*pi*fx)).*Vx;

end






